//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#ifndef YCSLogging_h
#define YCSLogging_h

NS_ASSUME_NONNULL_BEGIN

/// Log level.
typedef NS_ENUM(NSInteger, YCSLogLevel)
{
    /// Trace.
    YCSLogLevelTrace,
    /// Debug.
    YCSLogLevelDebug,
    /// Info.
    YCSLogLevelInformation,
    /// Warning.
    YCSLogLevelWarning,
    /// Error.
    YCSLogLevelError,
    /// Critical.
    YCSLogLevelCritical,
    /// None.
    YCSLogLevelNone
};


/// Log delegate.
///
/// Optionally implement this delegate to receive log events from the sdk. The instance can be set in ``YCSConnectOptions``.
@protocol YCSLogDelegate

/// Log message.
/// - Parameters:
///   - level: log level.
///   - message: log message.
///
/// Receives log events from the sdk.
- (void)log:(YCSLogLevel)level message:(NSString *)message;

@end

NS_ASSUME_NONNULL_END

#endif /* YCSLogging_h */
