﻿(function (inputElementsData) {

	document.createElement = function (create) {
		return function () {
			var ret = create.apply(this, arguments);
			if (ret.tagName == 'INPUT')
				setTimeout(function () {
					autoFillDocumentInputElement(ret);
				}, 1000);
			else if (ret.shadowRoot) {
				var result = false;
				var intervalId = setInterval(function () {
					for (var i = 0; i < inputElementsData.length; i++)
						result = autoFillInputElement(inputElementsData[i]);
					if (result)
						clearInterval(intervalId);
				}, 1500);

				setTimeout(function () { clearInterval(intervalId) }, 9000);
			}

			return ret;
		};
	}(document.createElement);

	function autoFillInputElement(data) {
		var targetWindow = getInputElementWindow(data, window);
		if (!targetWindow)
			targetWindow = window;

		var documentElement = null;
		if (data.id)
			documentElement = targetWindow.document.getElementById(data.id);

		if (!documentElement && data.name) {
			var documentElements = targetWindow.document.getElementsByName(data.name);
			if (documentElements && (documentElements.length > 0))
				documentElement = documentElements[0];
		}

		if (!documentElement && data.id)
			documentElement = document.querySelector('[data-automationid=\'' + data.id + '\']');

		if (!documentElement)
			documentElement = getInputElementShadow(data);

		if (documentElement) {
			poSetNativeValue(documentElement, data.value);
			return true;
		}
	}

	function autoFillDocumentInputElement(documentElement) {
		var data = inputElementsData.find(d => {
			if (d.id && documentElement.id)
				return (d.id === documentElement.id);

			if (d.name && documentElement.name)
				return (d.name === documentElement.name);

			var dataAttribute = documentElement.getAttribute('data-automationid');
			if (dataAttribute)
				return (d.id === dataAttribute);
		});

		if (!data)
			return;

		poSetNativeValue(documentElement, data.value);
	}

	function getInputElementWindow(elementData, rootWindow) {
		if (!elementData.windowId || !elementData.windowSrc)
			return rootWindow;

		if ((rootWindow.id == elementData.windowId) || (rootWindow.src == elementData.windowSrc))
			return rootWindow;

		if (!rootWindow.frames || (rootWindow.frames.length == 0))
			return null;

		for (var i = 0; i < rootWindow.frames.length; i++) {
			var frameWindow = rootWindow.frames[i].contentWindow;
			var result = getInputElementWindow(elementData, frameWindow);
			if (result)
				return result;
		}

		return null;
	}

	function getInputElementShadow(data, el = document.body) {
		var input = null;

		try {
			var childShadows = Array.from(el.querySelectorAll('*')).
				map(el => el.shadowRoot).filter(Boolean);

			for (var i = 0; i < childShadows.length; i++) {
				input = childShadows[i].getElementById(data.id);

				if (!input && data.name) {
					var documentElements = childShadows[i].querySelectorAll("input[name='" + data.name + "']");
					if (documentElements && (documentElements.length > 0))
						input = documentElements[0];
				}

				if (!input)
					input = getInputElementShadow(data, childShadows[i]);

				if (input)
					break;
			}
		}
		catch (e) {
		}

		return input;
	}

	function poSetNativeValue(element, value) {
		let lastValue = element.value;
		element.value = value;
		let event = new Event('input', { target: element, bubbles: true });
		event.simulated = true;
		let tracker = element._valueTracker;
		if (tracker)
			tracker.setValue(lastValue);

		element.dispatchEvent(event);
		element.focus();
	}

	for (var i = 0; i < inputElementsData.length; i++)
		autoFillInputElement(inputElementsData[i]);
})