grammar Done;

program: statement* EOF;

statement
	: expression
	;

expression
	: conditional_expression
	;
	
primary_expression
	: pe=primary_expression_start bracket_expression* ((member_access | method_invocation) bracket_expression*)*
	;

primary_expression_start
	: constant					#constantExpr
	| ID						#idExpr
	| OPAR expression CPAR		#parenExpr
	;

conditional_expression
	: null_coalescing_expression ('?' expression ':' expression)?
	;

null_coalescing_expression
	: conditional_or_expression ('??' (null_coalescing_expression | expression))?
	;

conditional_or_expression
	: conditional_and_expression (OP_OR conditional_and_expression)*
	;

conditional_and_expression
	: inclusive_or_expression (OP_AND inclusive_or_expression)*
	;

inclusive_or_expression
	: exclusive_or_expression ('|' exclusive_or_expression)*
	;

exclusive_or_expression
	: and_expression ('^' and_expression)*
	;

and_expression
	: equality_expression ('&' equality_expression)*
	;

equality_expression
	: relational_expression ((OP_EQ | OP_NE)  relational_expression)*
	;

relational_expression
	: additive_expression (('<' | '>' | '<=' | '>=') additive_expression)*
	;

additive_expression
	: multiplicative_expression (('+' | '-')  multiplicative_expression)*
	;

multiplicative_expression
	: unary_expression (('*' | '/' | '%')  unary_expression)*
	;

unary_expression
	: primary_expression
	| BANG unary_expression
	;

member_access
	: '?'? '.' ID
	;

bracket_expression
	: '?'? '[' argument_list ']'
	;

method_invocation: OPAR argument_list? CPAR;

argument_list: expression (COM expression)*;

constant: INT | STRING | BOOL | NULL;

BOOL: TRUE | FALSE;

TRUE: 'true';
FALSE: 'false';
NULL: 'null';
OBKT: '[';
CBKT: ']';
OPAR: '(';
CPAR: ')';
DOT: '.';
COM: ',';
SCOL: ';';
PLUS: '+';
MINUS: '-';
STAR: '*';
DIV: '/';
PERCENT: '%';
CARET: '^';
BANG: '!';
LT: '<';
GT: '>';
INTERR: '?';
OP_COALESCING: '??';
OP_AND: '&&';
OP_OR: '||';
OP_EQ: '==';
OP_NE: '!=';
LE: '<=';
GE: '>=';

ID: [a-zA-Z_][a-zA-Z0-9_]*;
INT: '0' | '-'?[1-9][0-9]*;
STRING: ('"' ~'"'* '"');
fragment COMMENT
	: '/*' .*? '*/'
	| '//' ~[\r\n]*
	;
WS: ([ \t\r\n]+ | COMMENT) -> Skip;