﻿(function () {
	var attachedInputElements = [];
	var targetInputTypes = ['email', 'number', 'password', 'search', 'tel', 'text', 'url', 'hidden'];
	var excludedInputIds = ['auth-captcha-guess', 'vendor-search-handler', 'code-id'];
	var inputData = [];
	var suppressExternalNotify = false;

	function attachAllChangeEvents(targetWindow) {
		suppressExternalNotify = true;
		var inputs = targetWindow.document.getElementsByTagName('input');
		for (var i = 0; i < inputs.length; i++)
			attachChangeEventsToInput(inputs[i]);

		for (var i = 0; i < targetWindow.frames.length; i++) {
			try {
				var frameWindow = targetWindow.frames[i].contentWindow;
				if (typeof frameWindow !== 'undefined')
					attachAllChangeEvents(frameWindow);
			}
			catch (e) {
			}
		}

		suppressExternalNotify = false;
		notifyExternal();
	}

	function attachAllChangeEventsShadow(targetWindow) {
		suppressExternalNotify = true;
		var inputs = querySelectorAllShadows('input', targetWindow);
		for (var i = 0; i < inputs.length; i++)
			attachChangeEventsToInput(inputs[i]);

		suppressExternalNotify = false;
		notifyExternal();
	}

	function querySelectorAllShadows(selector, el = document.body) {
		var childShadows = Array.from(el.querySelectorAll('*')).
			map(el => el.shadowRoot).filter(Boolean);

		var childResults = childShadows.map(child => querySelectorAllShadows(selector, child));
		var result = Array.from(el.querySelectorAll(selector));

		return result.concat(childResults).flat();
	}

	function attachChangeEventsToInput(input) {
		if (!targetInputTypes.includes(input.type) || excludedInputIds.includes(input.id) || attachedInputElements.includes(input))
			return;

		input.addEventListener('input', inputChangeHandler);
		input.addEventListener('change', inputChangeHandler);
		attachedInputElements.push(input);
		collectValue(input);
	}

	document.createElement = function (create) {
		return function () {
			var ret = create.apply(this, arguments);
			if (ret.tagName == 'INPUT')
				attachChangeEventsToInput(ret);

			return ret;
		};
	}(document.createElement);

	function inputChangeHandler(e) {
		collectValue(e.target);
	}

	function collectValue(element) {
		var doc = element.ownerDocument;
		var win = doc.defaultView || doc.parentWindow;
		var inputElement = element;

		var data = inputData.find(d => {
			if (d.id && inputElement.id)
				return (d.id === inputElement.id);

			if (d.name && inputElement.name)
				return (d.name === inputElement.name);

			var dataAttribute = inputElement.getAttribute('data-automationid');
			if (dataAttribute)
				return (d.id === dataAttribute);
		});

		if (typeof data === 'undefined') {
			data = {};
			inputData.push(data);
		}

		data.id = inputElement.id;
		data.name = inputElement.name;
		data.value = inputElement.value;
		data.type = inputElement.type;

		if (!data.id && !data.name) {
			var dataAttribute = inputElement.getAttribute('data-automationid');
			if (dataAttribute)
				data.id = dataAttribute;
		}

		if (win) {
			data.windowId = win.id;
			data.windowSrc = win.src;
		}

		notifyExternal();
	}

	function notifyExternal() {
		if (suppressExternalNotify)
			return;

		var notifyData = inputData.filter(d => {
			return (d.id || d.name);
		});

		if (notifyData && (notifyData.length > 0))
			window.mmtpocForm.collect(JSON.stringify(notifyData));
	}

	attachAllChangeEvents(window);
	document.addEventListener('click', (event) => { attachAllChangeEventsShadow(document); });
})();