//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#ifndef YCSCancellation_h
#define YCSCancellation_h

#include <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/// Operation canceled exception.
///
/// An exception that indicates an operation was canceled.
@interface YCSOperationCanceledException : NSException
@end

/// A cancellation ticket.
///
/// A special ticket that allows coordination of cancellation operations between a requestor and worker.
@interface YCSCancellationTicket : NSObject

/// Whether a cancellation has been requested.
@property (nonatomic, readonly, assign) BOOL isCancellationRequested;

/// Throws ``YCSOperationCanceledException`` if ``isCancellationRequested`` is set.
- (void)throwIfCancellationRequested;

/// Requests that the coordinated operation be canceled and causes ``isCancellationRequested`` to be set.
- (void)cancel;

/// An empty cancellation ticket that can never be canceled.
+ (YCSCancellationTicket *)none;

@end

NS_ASSUME_NONNULL_END

#endif /* YCSCancellation_h */
